"""
The note class provides a handy way of communicating in MOO, whether you want to leave somebody a quick message, or
perhaps leave a clue for solving an adventure. The note class can also provide a good starting base for more
specialized objects. The letter and newspaper objects are examples of this.
"""
