"""
The player class is the one of the basic classes of the virtual world. A player object is the same as any other object,
except that has its player bit set. This allows the server to connect a user to that object. A number of player
commands are defined as verbs of the `$player` class, as well as other useful functions.
"""
