# Generated by Django 4.2.7 on 2023-12-03 23:08

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import moo.core.models.acl


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("core", "0003_alter_verb_ability_alter_verb_method"),
    ]

    operations = [
        migrations.CreateModel(
            name="AccessibleObject",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("core.object", moo.core.models.acl.AccessibleMixin),
        ),
        migrations.CreateModel(
            name="AccessibleProperty",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("core.property", moo.core.models.acl.AccessibleMixin),
        ),
        migrations.CreateModel(
            name="AccessibleVerb",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("core.verb", moo.core.models.acl.AccessibleMixin),
        ),
        migrations.RemoveField(
            model_name="player",
            name="crypt",
        ),
        migrations.RemoveField(
            model_name="player",
            name="enabled",
        ),
        migrations.RemoveField(
            model_name="player",
            name="last_login",
        ),
        migrations.RemoveField(
            model_name="player",
            name="last_logout",
        ),
        migrations.RemoveField(
            model_name="player",
            name="session_id",
        ),
        migrations.AddField(
            model_name="player",
            name="user",
            field=models.OneToOneField(
                null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
            ),
        ),
    ]
