# Generated by Django 5.0.3 on 2024-07-05 22:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0010_object_obvious"),
    ]

    operations = [
        migrations.AlterField(
            model_name="object",
            name="name",
            field=models.CharField(db_index=True, max_length=255),
        ),
        migrations.AlterField(
            model_name="object",
            name="unique_name",
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AlterField(
            model_name="property",
            name="name",
            field=models.CharField(db_index=True, max_length=255),
        ),
        migrations.AlterField(
            model_name="verbname",
            name="name",
            field=models.CharField(db_index=True, max_length=255),
        ),
    ]
