#!moo verb emote --on $room --dspec any

# pylint: disable=return-outside-function,undefined-variable

"""
Handle the pose type of interaction with other players. It functions in a similar way to the :say verb,
but instead places the player's name at the front of the text. The actual output is done in two stages. The :emote verb
is responsible for telling the player the action s/he has just performed. The emote1 verb is then called to tell the
other objects in the room of the pose action. This provides a two stage mechanism; either or both of the verbs can be
overridden to provide special effects.
"""

from moo.core import context

if context.parser.words:
    message = " ".join(context.parser.words[1:])
else:
    message = " ".join(args)

context.caller.tell("You " + message)
this.emote1(message)
