"""
A container is a an object that can hold other object inside itself. This is similar to the idea of a room, except that
a container has no notion of exits, and can only hold things; it cannot hold players.
"""
